import os
import tkinter as tk
import tkinter.font as tkfont
import tkinter.scrolledtext as tkst
from operator import attrgetter

from search_outcome06 import SearchOutcome
from search_results06 import SearchResults

default_root_path = os.path.expanduser("~" + os.sep + "Documents")
search_options = ["file contents", "file names", "names & content"]
searchable_extensions = ["csv", "html", "java", "js", "py", "rtf", "txt", "xml"]
default_extensions = ["py", "txt"]
sort_options = ["matches", "weighting", "occurrences"]
print_depth = 1
debug = False


def readlines(path):
    """Return contents of text file as a list of lines."""
    with open(path) as file:
        try:
            return file.readlines()
        except UnicodeDecodeError:
            print(f" unicode error in {path}")
            return []


def index_and_search(dirpath, root_path_length, search_strings, depth, results, extensions):
    if debug:
        print(dirpath)
    try:
        filenames = os.listdir(dirpath)
    except PermissionError:
        return
    for filename in filenames:
        path = dirpath + os.sep + filename
        if os.path.isdir(path):
            if recursive.get():
                if depth < print_depth:
                    print(path)
                index_and_search(path, root_path_length, search_strings, depth + 1, results, extensions)
        else:
            if "." not in filename:
                continue
            extension = filename.split(".")[-1]
            if extension not in extensions:
                continue
            lines = []
            if selected.get() == "file contents" or selected.get() == "names & content":
                lines = readlines(path)
            if selected.get() == "file names" or selected.get() == "names & content":
                lines.insert(0, path)
            search_outcome = SearchOutcome(path, search_strings)
            for line in lines:
                if not case_sensitive.get():
                    line = line.lower()
                all_found = True
                for string in search_strings:
                    count = line.count(string)
                    if count > 0:
                        search_outcome.include(string, count)
                    else:
                        all_found = False
                if all_found:
                    search_outcome.add(line)
            search_matches, search_weighting, search_occurrences = search_outcome.get_summary(search_strings)
            if search_weighting > 0:
                results.add(search_outcome)
                if debug:
                    print(f"{search_occurrences:4} {path[root_path_length:]}")
            results.increment()
    if depth == 0:
        result_list = results.result_list
        # sorting - see https://docs.python.org/3/howto/sorting.html (Sort Stability and Complex Sorts)
        if sort_order.get() == "matches":
            result_list = sorted(result_list, key=attrgetter('occurrences'), reverse=True)
            result_list = sorted(result_list, key=attrgetter('weighting'), reverse=True)
            result_list = sorted(result_list, key=attrgetter('matches'), reverse=True)
        elif sort_order.get() == "weighting":
            result_list = sorted(result_list, key=attrgetter('occurrences'), reverse=True)
            result_list = sorted(result_list, key=attrgetter('matches'), reverse=True)
            result_list = sorted(result_list, key=attrgetter('weighting'), reverse=True)
        else:
            result_list = sorted(result_list, key=attrgetter('weighting'), reverse=True)
            result_list = sorted(result_list, key=attrgetter('matches'), reverse=True)
            result_list = sorted(result_list, key=attrgetter('occurrences'), reverse=True)
        results_txt["state"] = "normal"
        results_txt.delete("1.0", tk.END)
        contents = "FINAL RESULTS\n"
        for search_result in result_list:
            info = search_result.get_info(prune=root_path_length)
            contents += info + "\n"
        results_txt.insert(1.0, contents)
        results_txt["state"] = "disabled"
        output.set(f" {results.total_occurrences:,} occurrences in "
                   f"{len(results.result_list):,} / {results.total_files:,} files")


def respond_binding(event):
    respond()


def respond():
    search_string = search_input_txt.get()
    if not case_sensitive.get():
        search_string = search_string.lower()
    if not match_phrase.get():
        search_strings = search_string.split()
    else:
        search_strings = [search_string]
    root_path = root_input_txt.get()
    extensions = []
    for i, extension_state in enumerate(extension_states):
        if extension_state.get():
            extensions.append(searchable_extensions[i])
    print(f"using extensions {extensions}")
    results = SearchResults()
    if os.path.isdir(root_path):
        index_and_search(root_path, len(root_path) + 1, search_strings, 0, results, extensions)


window = tk.Tk()
window.geometry("830x600")
window.title("Search GUI")

default_font = tkfont.nametofont("TkDefaultFont")

default_font.configure(size=12, family="Arial")

row_counter = 0
lbl0 = tk.Label(window, text="Search", font=("Arial Bold", 20))
lbl0.grid(column=0, row=row_counter, sticky="W")
row_counter += 1

search_lbl = tk.Label(window, text="Search string:", font=("Arial Bold", 12))
search_lbl.grid(column=0, row=row_counter, sticky="W")
row_counter += 1

input = tk.StringVar(window)
search_input_txt = tk.Entry(window, width=30, textvariable=input, font=("Arial", 12))
search_input_txt.grid(column=0, row=row_counter, columnspan=2, sticky="W", padx=5, pady=5)
search_input_txt.bind('<Return>', respond_binding)

case_sensitive = tk.BooleanVar(window)
case_sensitive_chk = tk.Checkbutton(window, text="case sensitive", font=("Arial", 12), var=case_sensitive)
case_sensitive_chk.grid(column=2, row=row_counter, sticky="W")

match_phrase = tk.BooleanVar(window)
match_phrase_chk = tk.Checkbutton(window, text="match phrase", font=("Arial", 12), var=match_phrase)
match_phrase_chk.grid(column=3, row=row_counter, sticky="W")

row_counter += 1

root_lbl = tk.Label(window, text="Root dir:", font=("Arial Bold", 12))
root_lbl.grid(column=0, row=row_counter, sticky="W")
row_counter += 1

root_input = tk.StringVar(window)
root_input.set(default_root_path)
root_input_txt = tk.Entry(window, width=30, textvariable=root_input, font=("Arial", 12))
root_input_txt.grid(column=0, row=row_counter, columnspan=2, sticky="W", padx=5, pady=5)

recursive = tk.BooleanVar(window)
recursive.set(True)
recursive_chk = tk.Checkbutton(window, text="recursive", font=("Arial", 12), var=recursive)
recursive_chk.grid(column=2, row=row_counter, sticky="W")

row_counter += 1

type_lbl = tk.Label(window, text="Search type:", font=("Arial Bold", 12))
type_lbl.grid(column=0, row=row_counter, sticky="W")
row_counter += 1

selected = tk.StringVar(window)
selected.set(search_options[0])
for column_counter, option in enumerate(search_options):
    rad = tk.Radiobutton(window, text=option, value=option, variable=selected)
    rad.grid(column=column_counter, row=row_counter, sticky="W")
row_counter += 1

type_lbl = tk.Label(window, text="File extensions:", font=("Arial Bold", 12))
type_lbl.grid(column=0, row=row_counter, sticky="W")
row_counter += 1

column_counter = 0
extension_states = []
for extension in searchable_extensions:
    extension_state = tk.BooleanVar(window)
    if extension in default_extensions:
        extension_state.set(True)
    extension_states.append(extension_state)
    extension_chk = tk.Checkbutton(window, text=extension, font=("Arial", 12), var=extension_state)
    extension_chk.grid(column=column_counter, row=row_counter, sticky="W")
    column_counter += 1
    if column_counter == 4:
        column_counter = 0
        row_counter += 1
row_counter += 1

order_lbl = tk.Label(window, text="Sort order:", font=("Arial Bold", 12))
order_lbl.grid(column=0, row=row_counter, sticky="W")
row_counter += 1

sort_order = tk.StringVar(window)
sort_order.set(sort_options[0])
for column_counter, option in enumerate(sort_options):
    rad = tk.Radiobutton(window, text=option, value=option, variable=sort_order)
    rad.grid(column=column_counter, row=row_counter, sticky="W")
row_counter += 1

scrolled_text = False
if scrolled_text:
    results_txt = tkst.ScrolledText(window, width=100, height=12, state="disabled", wrap="none")
    results_txt.grid(column=0, row=row_counter, columnspan=5, sticky="W", padx=5, pady=5)
else:
    textContainer = tk.Frame(window, borderwidth=1, relief="sunken")
    results_txt = tk.Text(textContainer, width=100, height=12, state="disabled", wrap="none")
    results_txt_vsb = tk.Scrollbar(textContainer, orient="vertical", command=results_txt.yview)
    results_txt_hsb = tk.Scrollbar(textContainer, orient="horizontal", command=results_txt.xview)
    results_txt.configure(yscrollcommand=results_txt_vsb.set, xscrollcommand=results_txt_hsb.set)
    results_txt.grid(row=0, column=0, sticky="nsew")
    results_txt_vsb.grid(row=0, column=1, sticky="ns")
    results_txt_hsb.grid(row=1, column=0, sticky="ew")

    textContainer.grid_rowconfigure(0, weight=1)
    textContainer.grid_columnconfigure(0, weight=1)

    textContainer.grid(column=0, row=row_counter, columnspan=5, sticky="W", padx=5, pady=5)
row_counter += 1

output = tk.StringVar(window)
summary_txt = tk.Entry(window, width=60, textvariable=output, state="readonly", font=("Arial", 12))
summary_txt.grid(column=0, row=row_counter, columnspan=4, sticky="W", padx=5, pady=5)

window.mainloop()
